/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc:fr.bigeon.gclc.system.SystemConsoleManager.java
 * Created on: Dec 19, 2014
 */
package fr.bigeon.gclc.manager;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.nio.charset.Charset;

/** A console using the input stream and print stream.
 * <p>
 * The default constructor will use the system standart input and output.
 *
 * @author Emmanuel BIGEON */
public final class SystemConsoleManager implements ConsoleManager { // NOSONAR

    /** The default prompt */
    public static final String DEFAULT_PROMPT = "> "; //$NON-NLS-1$

    /** The command prompt. It can be changed. */
    private String prompt = DEFAULT_PROMPT;

    /** The print stream */
    private final PrintStream out;
    /** The input stream */
    private final BufferedReader in;

    /** If the manager is closed */
    private boolean closed = false;

    /** The prompting thread */
    private final Thread promptThread;

    /** The reading runnable */
    private final ReadingRunnable reading;

    /** This default constructor relies on the system defined standart output
     * and input stream. */
    public SystemConsoleManager() {
        this(System.out, System.in, Charset.defaultCharset());
    }

    /** @param out the output stream
     * @param in the input stream
     * @param charset the charset for the input */
    public SystemConsoleManager(PrintStream out, InputStream in,
            Charset charset) {
        super();
        this.out = out;
        this.in = new BufferedReader(new InputStreamReader(in, charset));
        reading = new ReadingRunnable(this.in);
        promptThread = new Thread(reading, "prompt"); //$NON-NLS-1$
        promptThread.setDaemon(true);
        promptThread.start();
    }

    /** @return the prompt */
    @Override
    public String getPrompt() {
        return prompt;
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.ConsoleManager#print(java.lang.Object) */
    @Override
    public void print(String object) throws IOException {
        checkOpen();
        out.print(object);
    }

    /** @throws IOException if the stream was closed */
    private void checkOpen() throws IOException {
        if (closed) {
            throw new IOException();
        }
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.ConsoleManager#println() */
    @Override
    public void println() throws IOException {
        checkOpen();
        out.println();
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.ConsoleManager#println(java.lang.Object) */
    @Override
    public void println(String object) throws IOException {
        checkOpen();
        out.println(object);
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.ConsoleManager#prompt() */
    @Override
    public String prompt() throws IOException {
        return prompt(prompt);
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.manager.ConsoleManager#prompt(long) */
    @Override
    public String prompt(long timeout) throws IOException {
        return prompt(prompt, timeout);
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.ConsoleManager#prompt(java.lang.String) */
    @Override
    public String prompt(String message) throws IOException {
        checkOpen();
        out.print(message);
        return reading.getMessage();
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.ConsoleManager#prompt(java.lang.String) */
    @Override
    public String prompt(String message, long timeout) throws IOException {
        checkOpen();
        out.print(message);
        return reading.getNextMessage(timeout);
    }

    /** @param prompt the prompt to set */
    @Override
    public void setPrompt(String prompt) {
        this.prompt = prompt;
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.manager.ConsoleManager#close() */
    @Override
    public void close() throws IOException {
        closed = true;
        reading.setRunning(false);
        promptThread.interrupt();
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.manager.ConsoleManager#isClosed() */
    @Override
    public boolean isClosed() {
        return closed;
    }

    /** Beware, in this implementation this is the same as closing the manager.
     * 
     * @see fr.bigeon.gclc.manager.ConsoleManager#interruptPrompt() */
    @Override
    public void interruptPrompt() {
        reading.interrupt();
    }

}
